### Python script to use SNAP as InSAR processor compatible with StaMPS PSI processing
# Author Jose Manuel Delgado Blasco
# Date: 21/06/2018
# Version: 1.0

# modify by dongyusen@gmail.com 2020.07.21



# Step 1 : preparing slaves in folder structure
# Step 2 : TOPSAR Splitting (Assembling) and Apply Orbit
# Step 3 : Coregistration and Interferogram generation
# Step 4 : StaMPS export

# Added option for CACHE and CPU specification by user
# Planned support for DEM selection and ORBIT type selection 


import os
from pathlib import Path
import sys
import glob
import subprocess
import shlex
import time
import shutil  
inputfile = sys.argv[1]
bar_message='\n#####################################################################\n'

# Getting configuration variables from inputfile
try:
    in_file = open(inputfile, 'r')
    for line in in_file.readlines():
        if "PROJECTFOLDER" in line:
            PROJECT = line.split('=')[1].strip()
            print(PROJECT)
        if "IW1" in line:
            IW = line.split('=')[1].strip()
            print(IW)
        if "MASTER" in line:
            MASTER = line.split('=')[1].strip()
            print(MASTER)
        if "GRAPHSFOLDER" in line:
            GRAPH = line.split('=')[1].strip()
            print(GRAPH)
        if "GPTBIN_PATH" in line:
            GPT = line.split('=')[1].strip()
            print(GPT)
        if "CACHE" in line:
            CACHE = line.split('=')[1].strip()
        if "CPU" in line:
            CPU = line.split('=')[1].strip()
finally:
        in_file.close()


finishedfile=''
finishedlist=[]
file1=[]
file2=[]



###################################################################################
##### StaMPS PSI export ##################
###################################################################################
coregfolder=PROJECT+'/pscoreg'
ifgfolder=PROJECT+'/psifg'
head, tail = os.path.split(MASTER)
print(tail)
logfolder=PROJECT+'/logs'


masterdate = tail[17:25]
# print('masterdate: ', masterdate)

outputexportfolder = PROJECT+'/PSINSAR_'+tail[17:25]
finishedfile=outputexportfolder+'/finished.txt'
finished_insar=PROJECT+'/psifg/finished.txt'

# print(finished_insar)

print('Checking finished files in /ifg')
with open(finished_insar, 'r') as f :
    for line in f.readlines():
        if masterdate == line[0:8].strip():
            file1.append(line.strip())

if not os.path.exists(outputexportfolder):
    os.makedirs(outputexportfolder)
if not os.path.exists(logfolder):
    os.makedirs(logfolder)
if not os.path.exists(finishedfile):
    Path(finishedfile).touch()


outlog=logfolder+'/export_proc_stdout.log'
out_file = open(outlog, 'a')
err_file=out_file
graphxml=GRAPH+'/sbas_export.xml'
graph2run=PROJECT+'/psgraphs/export2run.xml'
print(bar_message)
out_file.write(bar_message)
message='## StaMPS PSI export from sbas  started:\n'
print(message)
out_file.write(message)
print(bar_message)
out_file.write(bar_message)

k = 1
for filename in file1:
    ifgdim=Path(ifgfolder+'/'+filename)
    dimfile=Path(coregfolder+'/'+filename)
    if ifgdim.is_file() and dimfile.is_file():
        print('['+str(k)+'] Exporting pair: master-slave pair '+filename+'\n')

    with open(finishedfile, 'r') as file :
        filedata = file.read()
    if filename in filedata:
        print(' Alreay processed, Check InSAR_XXXX/finished.txt ..............')
        continue

    timeStarted = time.time()

    with open(graphxml, 'r') as file :
        filedata = file.read()
    filedata = filedata.replace('COREGFILE',coregfolder+'/'+filename)
    filedata = filedata.replace('IFGFILE', ifgfolder+'/'+filename)
    filedata = filedata.replace('OUTPUTFOLDER',outputexportfolder)
    with open(graph2run, 'w') as file:
        file.write(filedata)
    # args = [ GPT, graph2run, '-c', CACHE, '-q', CPU]
    args = [ GPT, graph2run, '-q', CPU]
    # print(args)
    # Launching process
    process = subprocess.Popen(args, stdout=subprocess.PIPE,stderr=subprocess.STDOUT)
    stdout = process.communicate()[0]
    print('SNAP STDOUT:{}'.format(stdout))
    timeDelta = time.time() - timeStarted     # Get execution time.
    print(('['+str(k)+'] Finished process in '+str(timeDelta/60.0)+' mins.'))

    if process.returncode != 0 :
        message='Error exporting '+filename+'\n' 

    else:
        message='Stamps export of '+filename+' successfully completed.\n'
        print(message)

        with open(finishedfile, 'a') as file :
            file.write(filename+'\n')
    if os.path.isfile('target.dim'):
        shutil.rmtree('target.data') 
        os.remove('target.dim')
 
    print(bar_message) 
    k = k + 1   



print('-------------Finish exporting')
print('-------------First, run mt_prep_snap ' +  masterdate  +'  '+  outputexportfolder  + ' 0.4 .......' + ' in ' + outputexportfolder)
print('-------------Then run ps_parms_initial.m  in matlab')
print('               Good luck  :)                    ')
